package org.fuseri.moduleexercise.exercise;

import org.fuseri.model.dto.common.Result;
import org.fuseri.moduleexercise.question.Question;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

/**
 * A repository interface for managing Exercise entities
 */
@Repository
public interface ExerciseRepository extends JpaRepository<Exercise, Long> {

    /**
     * Filters the exercises by the specified difficulty level and course id,
     * and returns a {@link Result} object containing these filtered exercises
     * along with pagination information
     *
     * @param pageRequest the pagination settings for the result
     * @param courseId    the id of the course to filter by
     * @param difficulty  the difficulty level to filter by
     * @return a {@link Result} object containing a list of paginated exercises that match the filter criteria
     */
    @Query("SELECT e FROM Exercise e WHERE e.courseId = :courseId AND e.difficulty = :difficulty")
    Page<Exercise> filterPerDifficultyPerCourse(PageRequest pageRequest, long courseId, int difficulty);

    @Query("SELECT q FROM Exercise e JOIN e.questions q WHERE e.id = :exerciseId")
    Page<Question> getQuestions(PageRequest pageRequest, @Param("exerciseId") Long exerciseId);

    @Modifying
    @Transactional
    @Query(value = "TRUNCATE Table exercise",nativeQuery = true)
    void resetTable();

    @Modifying
    @Transactional
    @Query( value = "ALTER TABLE Exercise ALTER COLUMN id RESTART WITH 1",nativeQuery = true)
    void resetId();


}
