package org.fuseri.moduleexercise.question;

import jakarta.persistence.EntityNotFoundException;
import lombok.Getter;
import org.fuseri.moduleexercise.common.DomainService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * Represent a service for managing Question entities
 */
@Service
public class QuestionService extends DomainService<Question> {

    /**
     * The repository instance used by this service
     */
    @Getter
    private final QuestionRepository repository;

    /**
     * Construct a new instance of QuestionService with the specified repository
     *
     * @param repository the repository instance to be used by this service
     */
    @Autowired
    public QuestionService(QuestionRepository repository) {
        this.repository = repository;
    }

    /**
     * Retrieve the Question entity with the specified ID
     *
     * @param id the ID of the Question entity to retrieve
     * @return the Question entity with the specified ID
     * @throws EntityNotFoundException if no Question entity exists with the specified ID
     */
    @Transactional(readOnly = true)
    public Question find(long id) {
        return repository.findById(id)
                .orElseThrow(() -> new EntityNotFoundException("Question '" + id + "' not found."));
    }

    //@TestOnly
    @Transactional
    public void resetId() {
        repository.resetId();
    }

}
